/*---------------------------------------------------------------------------
Program :	QuickMenu	Version 1.3
Author :	Michel Leunen
Email :		michel@leunen.com
Date :		2003
Description :
*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <shellapi.h>
#include <memory>
#include "Unit1.h"
#include "Unit2.h"
#include "qmres.rh"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
  OldListBoxWindowProc=FileListBox->WindowProc;
  FileListBox->WindowProc=NewListBoxWindowProc;
  ::DragAcceptFiles(FileListBox->Handle,true);
  SHFILEINFO info;
  DWORD ImageHandle = SHGetFileInfo("",
                                    0,
                                    &info,
                                    sizeof(info),
                                    SHGFI_ICON |
                                    SHGFI_SHELLICONSIZE |
                                    SHGFI_SMALLICON |
                                    SHGFI_SYSICONINDEX);

  if (ImageHandle != 0)
  {
    ImageList->Handle=ImageHandle;
    ImageList->ShareImages=true;
  }
  LoadDefaultParams();
  std::auto_ptr<Graphics::TIcon> newIcon(new Graphics::TIcon());
  try
  {
    if(IconCheckBox->Checked)
    {
      newIcon->Handle=LoadIcon(HInstance,MAKEINTRESOURCE(ID_SYSTRAYICON));
      if(newIcon->Handle==NULL)
      {
        Application->MessageBox("Icon not found","Resource Error",MB_OK);
      }
    }
    else newIcon->LoadFromFile(IconEdit->Text);
  }
  catch(EInOutError &exception)
  {
    Application->MessageBox("Icon not found","Resource Error",MB_OK);
    Application->Terminate();
  }
  SysTray=new TSysTray(this);
  SysTray->PopupMenu=PopupMenu;
  SysTray->Icon=newIcon.get();
  SysTray->TrayHint="QuickMenu";
  SysTray->AddIcon();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
  Action=caNone;
  ShowWindow(Application->Handle,SW_HIDE);
  Visible=false;
  SaveDefaultParams();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Configure1Click(TObject *Sender)
{
  //show window
	ShowWindow(Application->Handle,SW_SHOW);
  Visible=true;
  SetForegroundWindow(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Exit1Click(TObject *Sender)
{
  if(MessageBox(Handle,"Are you sure you want to quit?",
                "QuickMenu WARNING",
                MB_YESNO | MB_ICONQUESTION | MB_APPLMODAL)==IDYES)
  {
    SysTray->RemoveIcon();
    SaveDefaultParams();
    ::DragAcceptFiles(FileListBox->Handle,false);
    Application->Terminate();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::LoadDefaultParams()
{
  HKEY hKey;
  if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Michel Leunen\\QuickMenu\\",
      0,KEY_READ,&hKey)==ERROR_SUCCESS)
  {
    Top=RegReadInteger(hKey,"TopPosition");
    Left=RegReadInteger(hKey,"LeftPosition");
    LaunchCheckBox->Checked=RegReadInteger(hKey,"LaunchAtStartup");
    IconCheckBox->Checked=RegReadInteger(hKey,"UseDefaultIcon");
    IconEdit->Text=RegReadString(hKey,"IconPath");
    IconEdit->Enabled=!IconCheckBox->Checked;
    BrowseButton->Enabled=!IconCheckBox->Checked;
    IconLabel->Enabled=!IconCheckBox->Checked;
    int count=RegReadInteger(hKey,"ItemCount");;
    RegCloseKey(hKey);
    if(count!=0)
    {
      FileListBox->Clear();
      for(int i=0;i<count;++i)
      {
        MenuItem mi;
        AnsiString key="Software\\Michel Leunen\\QuickMenu\\Item"+IntToStr(i);
        if(RegOpenKeyEx(HKEY_CURRENT_USER,key.c_str(),
            0,KEY_READ,&hKey)==ERROR_SUCCESS)
        {
          mi.Path=RegReadString(hKey,"Path");
          mi.Name=RegReadString(hKey,"Name");
          mi.Param=RegReadString(hKey,"Parameter");
          ol.Add(mi);
          FileListBox->Items->Add(mi.Name);
          RegCloseKey(hKey);
        }
      }
      RecreateMenu();
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SaveDefaultParams()
{
  HKEY hKey;
  DWORD disp;
  if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\Michel Leunen\\QuickMenu",
      0,NULL,REG_OPTION_NON_VOLATILE,KEY_WRITE,NULL,&hKey,&disp)==ERROR_SUCCESS)
  {
    RegSetValueEx(hKey,"TopPosition",0,REG_DWORD,(BYTE*)&Top,sizeof(int));
    RegSetValueEx(hKey,"LeftPosition",0,REG_DWORD,(BYTE*)&Left,sizeof(int));
    int dummy=LaunchCheckBox->Checked;
    RegSetValueEx(hKey,"LaunchAtStartup",0,REG_DWORD,(BYTE*)&dummy,sizeof(int));
    dummy=IconCheckBox->Checked;
    RegSetValueEx(hKey,"UseDefaultIcon",0,REG_DWORD,(BYTE*)&dummy,sizeof(int));
    RegSetValueEx(hKey,"IconPath",0,REG_SZ,(BYTE*)IconEdit->Text.c_str(),
        IconEdit->Text.Length());
    int count=ol.Count();
    RegSetValueEx(hKey,"ItemCount",0,REG_DWORD,(BYTE*)&count,sizeof(int));
    RegCloseKey(hKey);
    if(count!=0)
    {
      for(int i=0;i<count;++i)
      {
        AnsiString key="Software\\Michel Leunen\\QuickMenu\\Item"+IntToStr(i);
        if(RegCreateKeyEx(HKEY_CURRENT_USER,key.c_str(),
            0,NULL,REG_OPTION_NON_VOLATILE,KEY_WRITE,NULL,&hKey,&disp)==ERROR_SUCCESS)
        {
          RegSetValueEx(hKey,"Path",0,REG_SZ,(BYTE*)ol[i].Path.c_str(),
              ol[i].Path.Length());
          RegSetValueEx(hKey,"Name",0,REG_SZ,(BYTE*)ol[i].Name.c_str(),
              ol[i].Name.Length());
          RegSetValueEx(hKey,"Parameter",0,REG_SZ,(BYTE*)ol[i].Param.c_str(),
              ol[i].Param.Length());
          RegCloseKey(hKey);
        }
      }
    }
  }
  if(RegOpenKeyEx(HKEY_CURRENT_USER,
      "Software\\Microsoft\\Windows\\CurrentVersion\\Run",0,KEY_WRITE,&hKey)
          ==ERROR_SUCCESS)
  {
    if(int dummy=LaunchCheckBox->Checked)
    {
      RegSetValueEx(hKey,"LaunchAtStartup",0,REG_DWORD,(BYTE*)&dummy,sizeof(int));
      RegSetValueEx(hKey,"QuickMenu",0,REG_SZ,(BYTE*)Application->ExeName.c_str(),
          Application->ExeName.Length());
    }
    else RegDeleteValue(hKey,"QuickMenu");
    RegCloseKey(hKey);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::NewListBoxWindowProc(TMessage &Msg)
{
  if(Msg.Msg==WM_DROPFILES)
  {
    int NumFiles=DragQueryFile((HDROP)Msg.WParam,0xFFFFFFFF,(LPSTR)NULL,NULL);
    for(int i=0;i<NumFiles;i++)
    {
      int NameLength=DragQueryFile((HDROP)Msg.WParam,i,NULL,NULL);
      char *FileName=new char[NameLength+1];
      DragQueryFile((HDROP)Msg.WParam,i,FileName,NameLength+1);
      AddMenuItem(FileName);
      delete[] FileName;
    }
  }
  OldListBoxWindowProc(Msg);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::AddMenuItem(AnsiString item)
{
  MenuItem mi;
  Form2->PathEdit->Text=item;
  Form2->NameEdit->Text=item;
  Form2->ParamEdit->Text="";
  if(Form2->ShowModal()==mrOk)
  {
    mi.Path=Form2->PathEdit->Text;
    mi.Name=Form2->NameEdit->Text;
    mi.Param=Form2->ParamEdit->Text;
    if(FileListBox->ItemIndex==-1)
    {
      FileListBox->Items->Add(Form2->NameEdit->Text);
      ol.Add(mi);
    }
    else
    {
      ol.Insert(FileListBox->ItemIndex,mi);
      FileListBox->Items->Insert(FileListBox->ItemIndex,Form2->NameEdit->Text);
    }
    int extent=0;
    for(int x=0;x<FileListBox->Items->Count; x++)
    {
      int size =FileListBox->Canvas->TextWidth(FileListBox->Items->Strings[x]);
      if(size>extent)extent=size;
    }
    FileListBox->Perform(LB_SETHORIZONTALEXTENT,(WPARAM)extent+10,0);
    RecreateMenu();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SeparatorBtnClick(TObject *Sender)
{
  AddMenuItem("---(Separator)---");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RenameBtnClick(TObject *Sender)
{
  if(FileListBox->ItemIndex!=-1)
  {
    MenuItem mi=ol[FileListBox->ItemIndex];
    Form2->PathEdit->Text=mi.Path;
    Form2->NameEdit->Text=mi.Name;
    Form2->ParamEdit->Text=mi.Param;
    if(Form2->ShowModal()==mrOk)
    {
      mi.Path=Form2->PathEdit->Text;
      mi.Name=Form2->NameEdit->Text;
      mi.Param=Form2->ParamEdit->Text;
      ol[FileListBox->ItemIndex]=mi;
      FileListBox->Items->Strings[FileListBox->ItemIndex]=Form2->NameEdit->Text;
    }
  }
  RecreateMenu();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RecreateMenu()
{
  //Delete all menu items
  int count=PopupMenu->Items->Count;
  if(count>3)
  {
    for(int i=0;i<count-3;++i)
        PopupMenu->Items->Delete(0);
  }
  //Recreate it
  for(unsigned int i=0;i<ol.Count();++i)
  {
    TMenuItem *tmpmi=new TMenuItem(this);
    if(ol[i].Name=="---(Separator)---")tmpmi->Caption="-";
    else
    {
      tmpmi->Caption=ol[i].Name;
      tmpmi->OnClick=MenuOnClick;
    }
    tmpmi->Tag=i;
    SHFILEINFO info;
    DWORD res=SHGetFileInfo(ol[i].Path.c_str(),
                            0,&info,sizeof(info),
                            SHGFI_ICON | SHGFI_SHELLICONSIZE |
                            SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
    if(res!=0)
    {
      //use Bitmap property to keep the transparency 
      ImageList->GetBitmap(info.iIcon,tmpmi->Bitmap);
    }
    PopupMenu->Items->Insert(PopupMenu->Items->Count-3,tmpmi);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::AddBtnClick(TObject *Sender)
{
  AddMenuItem("");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RemoveBtnClick(TObject *Sender)
{
  if(FileListBox->ItemIndex!=-1)
  {
    ol.Remove(FileListBox->ItemIndex);
    FileListBox->Clear();
    for(unsigned int i=0;i<ol.Count();++i)FileListBox->Items->Add(ol[i].Name);
    RecreateMenu();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::MoveUpBtnClick(TObject *Sender)
{
  int index=FileListBox->ItemIndex;
  if(index>0 )
  {
    ol.MoveUp(index);
    FileListBox->Clear();
    for(unsigned int i=0;i<ol.Count();++i)FileListBox->Items->Add(ol[i].Name);
    FileListBox->ItemIndex=index-1;
    RecreateMenu();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::MoveDownBtnClick(TObject *Sender)
{
  int index=FileListBox->ItemIndex;
  if(index!=-1 && index!=FileListBox->Items->Count)
  {
    ol.MoveDown(index);
    FileListBox->Clear();
    for(unsigned int i=0;i<ol.Count();++i)FileListBox->Items->Add(ol[i].Name);
    FileListBox->ItemIndex=index+1;
    RecreateMenu();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::MenuOnClick(TObject *Sender)
{
  TMenuItem *mi=dynamic_cast<TMenuItem*>(Sender);
  if(mi)
  {
    ShellExecute(GetDesktopWindow(),"open",
        ol[mi->Tag].Path.c_str(),ol[mi->Tag].Param.c_str(),NULL,SW_SHOW);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::IconCheckBoxClick(TObject *Sender)
{
  IconEdit->Enabled=!IconCheckBox->Checked;
  BrowseButton->Enabled=!IconCheckBox->Checked;
  IconLabel->Enabled=!IconCheckBox->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::BrowseButtonClick(TObject *Sender)
{
  OpenDialog->InitialDir=ExtractFilePath(Application->ExeName);
  if(OpenDialog->Execute())
  {
    IconEdit->Text=OpenDialog->FileName;
  }
}
//---------------------------------------------------------------------------
int __fastcall TForm1::RegReadInteger(HKEY hkey,AnsiString key)
{
  DWORD size=sizeof(int);
  int value;
  RegQueryValueEx(hkey,key.c_str(),NULL,NULL,(BYTE*)&value,&size);
  return value;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TForm1::RegReadString(HKEY hkey,AnsiString key)
{
  DWORD size=1;
  RegQueryValueEx(hkey,key.c_str(),NULL,NULL,NULL,&size);
  char *value=new char[size];
  RegQueryValueEx(hkey,key.c_str(),NULL,NULL,(BYTE*)value,&size);
  AnsiString ret;
  if(size==0)ret="";
  else ret=AnsiString(value);
  delete[] value;
  return ret;
}
//---------------------------------------------------------------------------
